@@ -93,13 +93,8 @@ decode_gl <- function(polyline, precision=6, forceline=TRUE) {
 #' @export
 #' @examples \dontrun{
 #'   exroutes <- viaroute(50, 0, 51, 1)
-#'   r1 <- viaroute2sldf(exroutes)
-#'   exroutes <- viaroute(50, 0, 51, 1, zoom = 4)
-#'   r2 <- viaroute2sldf(exroutes)
-#'   # Requires mapview to be installed
-#'   # Show the difference between outputs with different zoom levels
-#'   mapview::mapview(r1) +
-#'    mapview::mapview(r2, color = "blue")
+#'   r <- viaroute2sldf(exroutes)
+#'   plot(r)
 #'   exroutes <- viaroute(viapoints=list(data.frame(x=c(-33.5,-33.6,-33.7),y=c(150,150.1,150.2))))
 #'   r <- viaroute2sldf(exroutes)
 #'   plot(r)
@@ -126,25 +121,47 @@ viaroute <- function(startlat = NULL, startlng = NULL, endlat = NULL,
 
   if (missing(viapoints) == FALSE) {
 
-    if (is(viapoints,"list")) {
+    if (!is(viapoints,"list")) {
       stop("viapoints is not a list.")
     }
 
     i <- 1
     while (i <= length(viapoints)) {
       if(ncol(viapoints[[i]]) == 3) {
-        returnval[i] <- gsub('\\\\\\\\\"','\\\\\\"',gsub('\\\\','\\\\\\\\',RCurl::getURL(paste0(qryurl,"loc=",paste0(viapoints[[i]][,1],',',viapoints[[i]][,2],'&u=',viapoints[[i]][,3],collapse='&loc='),'&',
-                             paste0(paste0(
-                             c("z","instructions","alt","geometry","uturns"),'=',
-                             c(zoom,instructions,alt,geometry,uturns)),collapse='&')
-        ))))
+        if (api == 4) {
+          returnval[i] <- gsub('\\\\\\\\\"','\\\\\\"',gsub('\\\\','\\\\\\\\',RCurl::getURL(paste0(qryurl,"loc=",paste0(viapoints[[i]][,1],',',viapoints[[i]][,2],'&u=',viapoints[[i]][,3],collapse='&loc='),'&',
+                               paste0(paste0(
+                               c("z","instructions","alt","geometry","uturns"),'=',
+                               c(zoom,instructions,alt,geometry,uturns)),collapse='&')
+          ))))
+        } else {
+          returnval[i] <- gsub('\\\\','\\\\\\\\',RCurl::getURL(
+            paste0(qryurl,
+                   paste0(viapoints[[i]][,2],',',viapoints[[i]][,1],collapse=';'),
+                   "?overview=full&",
+                   paste0(paste0(c("steps","alternatives","continue_straight"),'=',
+                                 c(instructions,alt,uturns)), collapse='&')
+            )
+          ))
+        }
       }
       else {
-        returnval[i] <- gsub('\\\\\\\\\"','\\\\\\"',gsub('\\\\','\\\\\\\\',RCurl::getURL(paste0(qryurl,"loc=",paste0(paste0(viapoints[[i]][,1],',',viapoints[[i]][,2]),collapse='&loc='),'&',
-           paste0(paste0(
-             c("z","instructions","alt","geometry","uturns"),'=',
-             c(zoom,instructions,alt,geometry,uturns)),collapse='&')
-        ))))
+        if (api == 4) {
+          returnval[i] <- gsub('\\\\\\\\\"','\\\\\\"',gsub('\\\\','\\\\\\\\',RCurl::getURL(paste0(qryurl,"loc=",paste0(paste0(viapoints[[i]][,1],',',viapoints[[i]][,2]),collapse='&loc='),'&',
+             paste0(paste0(
+               c("z","instructions","alt","geometry","uturns"),'=',
+               c(zoom,instructions,alt,geometry,uturns)),collapse='&')
+          ))))
+        } else {
+          returnval[i] <- gsub('\\\\','\\\\\\\\',RCurl::getURL(
+            paste0(qryurl,
+                   paste0(viapoints[[i]][,2],',',viapoints[[i]][,1],collapse=';'),
+                   "?overview=full&",
+                   paste0(paste0(c("steps","alternatives","continue_straight"),'=',
+                                 c(instructions,alt,uturns)), collapse='&')
+            )
+          ))
+        }
       }
       i <- i + 1
     }
@@ -196,6 +213,24 @@ viaroute <- function(startlat = NULL, startlng = NULL, endlat = NULL,
   else {
     stop("Missing viapoints coordinates")
   }
+  if (length(returnval) > 1) {
+    removerowlist <- c()
+    for (i in 1:length(returnval)) {
+      if (returnval[i] == "") {
+        removerowlist <- c(removerowlist,i)
+      }
+    }
+    if (length(removerowlist) == length(returnval)) {
+      stop("OSRM server retruned empty result for all routes")
+    } else {
+      warning(paste0("Routes ",paste0(removerowlist, collapse=', '), " returned empty result, removing from result"))
+      returnval <- returnval[-removerowlist]
+    }
+  } else {
+    if (returnval == "") {
+      stop("OSRM server returned empty result")
+    }
+  }
 
   return(returnval)
 
@@ -688,4 +723,4 @@ table2matrix <- function(lat, lng, destlat = NA, destlng = NA,
     return(tabledata2$distance_table)
   }
 
-}
\ No newline at end of file
+}
